# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....claims.types.claim_status import ClaimStatus
from ....commons.types.encounter_external_id import EncounterExternalId
from ....commons.types.encounter_id import EncounterId
from ....commons.types.page_token import PageToken
from ....commons.types.work_queue_id import WorkQueueId
from ....encounters_universal.types.universal_encounter_create import UniversalEncounterCreate
from ....encounters_universal.types.universal_encounter_create_from_pre_encounter import (
    UniversalEncounterCreateFromPreEncounter,
)
from ....encounters_universal.types.universal_encounter_update import UniversalEncounterUpdate
from ....tags.types.tag_id import TagId
from .raw_client import AsyncRawV4Client, RawV4Client
from .types.billable_status_type import BillableStatusType
from .types.encounter import Encounter
from .types.encounter_create import EncounterCreate
from .types.encounter_create_from_pre_encounter import EncounterCreateFromPreEncounter
from .types.encounter_owner_of_next_action_type import EncounterOwnerOfNextActionType
from .types.encounter_page import EncounterPage
from .types.encounter_sort_options import EncounterSortOptions
from .types.encounter_update import EncounterUpdate
from .types.responsible_party_type import ResponsiblePartyType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V4Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV4Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV4Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV4Client
        """
        return self._raw_client

    def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        sort: typing.Optional[EncounterSortOptions] = None,
        page_token: typing.Optional[PageToken] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        primary_payer_names: typing.Optional[str] = None,
        search_term: typing.Optional[str] = None,
        external_id: typing.Optional[EncounterExternalId] = None,
        diagnoses_updated_since: typing.Optional[dt.datetime] = None,
        tag_ids: typing.Optional[typing.Union[TagId, typing.Sequence[TagId]]] = None,
        work_queue_id: typing.Optional[WorkQueueId] = None,
        billable_status: typing.Optional[BillableStatusType] = None,
        responsible_party: typing.Optional[ResponsiblePartyType] = None,
        owner_of_next_action: typing.Optional[EncounterOwnerOfNextActionType] = None,
        patient_external_id: typing.Optional[str] = None,
        include_merged_patient_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EncounterPage:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        claim_status : typing.Optional[ClaimStatus]
            Indicates the current status of an insurance claim within the billing process.

        sort : typing.Optional[EncounterSortOptions]
            Defaults to created_at:desc.

        page_token : typing.Optional[PageToken]

        date_of_service_min : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.

        date_of_service_max : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.

        primary_payer_names : typing.Optional[str]
            Comma delimited string.

        search_term : typing.Optional[str]
            Filter by any of the following fields: encounter_id, claim_id, patient external_id,
            patient date of birth, patient first name, patient last name,
            or encounter external id.

        external_id : typing.Optional[EncounterExternalId]
            Filter to an exact match on encounter external_id, if one exists.

        diagnoses_updated_since : typing.Optional[dt.datetime]
            ISO 8601 timestamp; ideally in UTC (although not required): 2019-08-24T14:15:22Z.

        tag_ids : typing.Optional[typing.Union[TagId, typing.Sequence[TagId]]]
            Filter by name of tags on encounters.

        work_queue_id : typing.Optional[WorkQueueId]

        billable_status : typing.Optional[BillableStatusType]
            Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.

        responsible_party : typing.Optional[ResponsiblePartyType]
            Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.

        owner_of_next_action : typing.Optional[EncounterOwnerOfNextActionType]
            The party who is responsible for taking the next action on an Encounter, as defined by ownership of open Tasks.

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        include_merged_patient_data : typing.Optional[bool]
            If true and patient_external_id is set, then also include the encounters of all alternative patients.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EncounterPage

        Examples
        --------
        import datetime

        from candid import CandidApiClient
        from candid.resources.claims import ClaimStatus
        from candid.resources.encounters.resources.v_4 import EncounterSortOptions

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.encounters.v_4.get_all(
            limit=100,
            claim_status=ClaimStatus.BILLER_RECEIVED,
            sort=EncounterSortOptions.CREATED_AT_ASC,
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
            date_of_service_min=datetime.date.fromisoformat(
                "2019-08-24",
            ),
            date_of_service_max=datetime.date.fromisoformat(
                "2019-08-25",
            ),
            primary_payer_names="Medicare,Medicaid",
            search_term="doe",
            external_id="123456",
            diagnoses_updated_since=datetime.datetime.fromisoformat(
                "2019-08-24 14:15:22+00:00",
            ),
        )
        """
        _response = self._raw_client.get_all(
            limit=limit,
            claim_status=claim_status,
            sort=sort,
            page_token=page_token,
            date_of_service_min=date_of_service_min,
            date_of_service_max=date_of_service_max,
            primary_payer_names=primary_payer_names,
            search_term=search_term,
            external_id=external_id,
            diagnoses_updated_since=diagnoses_updated_since,
            tag_ids=tag_ids,
            work_queue_id=work_queue_id,
            billable_status=billable_status,
            responsible_party=responsible_party,
            owner_of_next_action=owner_of_next_action,
            patient_external_id=patient_external_id,
            include_merged_patient_data=include_merged_patient_data,
            request_options=request_options,
        )
        return _response.data

    def get(self, encounter_id: EncounterId, *, request_options: typing.Optional[RequestOptions] = None) -> Encounter:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.encounters.v_4.get(
            encounter_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get(encounter_id, request_options=request_options)
        return _response.data

    def create_universal(
        self, *, request: UniversalEncounterCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> Encounter:
        """
        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        request : UniversalEncounterCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import datetime

        from candid import CandidApiClient
        from candid.resources.commons import (
            State,
            StreetAddressLongZip,
            StreetAddressShortZip,
        )
        from candid.resources.encounter_providers.resources.v_2 import BillingProvider
        from candid.resources.encounters.resources.v_4 import (
            BillableStatusType,
            EncounterSubmissionExpectation,
            ResponsiblePartyType,
        )
        from candid.resources.encounters_universal import UniversalEncounterCreate
        from candid.resources.individual import Gender, PatientCreate

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.encounters.v_4.create_universal(
            request=UniversalEncounterCreate(
                billing_provider=BillingProvider(
                    address=StreetAddressLongZip(
                        zip_plus_four_code="zip_plus_four_code",
                        address_1="address1",
                        city="city",
                        state=State.AA,
                        zip_code="zip_code",
                    ),
                    tax_id="tax_id",
                    npi="npi",
                ),
                submission_expectation=EncounterSubmissionExpectation.TARGET_PROFESSIONAL,
                patient=PatientCreate(
                    external_id="external_id",
                    date_of_birth=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    address=StreetAddressShortZip(
                        address_1="address1",
                        city="city",
                        state=State.AA,
                        zip_code="zip_code",
                    ),
                    first_name="first_name",
                    last_name="last_name",
                    gender=Gender.MALE,
                ),
                responsible_party=ResponsiblePartyType.INSURANCE_PAY,
                external_id="external_id",
                patient_authorized_release=True,
                benefits_assigned_to_provider=True,
                provider_accepts_assignment=True,
                billable_status=BillableStatusType.BILLABLE,
            ),
        )
        """
        _response = self._raw_client.create_universal(request=request, request_options=request_options)
        return _response.data

    def create(self, *, request: EncounterCreate, request_options: typing.Optional[RequestOptions] = None) -> Encounter:
        """
        Parameters
        ----------
        request : EncounterCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import datetime

        from candid import CandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            State,
            StreetAddressLongZip,
            StreetAddressShortZip,
        )
        from candid.resources.diagnoses import DiagnosisCreate, DiagnosisTypeCode
        from candid.resources.encounter_providers.resources.v_2 import (
            BillingProvider,
            RenderingProvider,
        )
        from candid.resources.encounters.resources.v_4 import (
            BillableStatusType,
            EncounterCreate,
            ResponsiblePartyType,
        )
        from candid.resources.individual import Gender, PatientCreate

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.encounters.v_4.create(
            request=EncounterCreate(
                billing_provider=BillingProvider(
                    address=StreetAddressLongZip(
                        zip_plus_four_code="zip_plus_four_code",
                        address_1="address1",
                        city="city",
                        state=State.AA,
                        zip_code="zip_code",
                    ),
                    tax_id="tax_id",
                    npi="npi",
                ),
                diagnoses=[
                    DiagnosisCreate(
                        code_type=DiagnosisTypeCode.ABF,
                        code="code",
                    ),
                    DiagnosisCreate(
                        code_type=DiagnosisTypeCode.ABF,
                        code="code",
                    ),
                ],
                place_of_service_code=FacilityTypeCode.PHARMACY,
                rendering_provider=RenderingProvider(
                    npi="npi",
                ),
                patient=PatientCreate(
                    external_id="external_id",
                    date_of_birth=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    address=StreetAddressShortZip(
                        address_1="address1",
                        city="city",
                        state=State.AA,
                        zip_code="zip_code",
                    ),
                    first_name="first_name",
                    last_name="last_name",
                    gender=Gender.MALE,
                ),
                responsible_party=ResponsiblePartyType.INSURANCE_PAY,
                external_id="external_id",
                patient_authorized_release=True,
                benefits_assigned_to_provider=True,
                provider_accepts_assignment=True,
                billable_status=BillableStatusType.BILLABLE,
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def create_from_pre_encounter_patient_universal(
        self,
        *,
        request: UniversalEncounterCreateFromPreEncounter,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Encounter:
        """
        Create an encounter from a pre-encounter patient and appointment. This endpoint is intended to be used by consumers who are managing
        patients and appointments in the pre-encounter service and is currently under development. Consumers who are not taking advantage
        of the pre-encounter service should use the standard create endpoint.

        The endpoint will create an encounter from the provided fields, pulling information from the provided patient and appointment objects
        where applicable. In particular, the following fields are populated from the patient and appointment objects:
          - Patient
          - Referring Provider
          - Subscriber Primary
          - Subscriber Secondary
          - Referral Number
          - Responsible Party
          - Guarantor

        Utilizing this endpoint opts you into automatic updating of the encounter when the patient or appointment is updated, assuming the
        encounter has not already been submitted or adjudicated.

        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        request : UniversalEncounterCreateFromPreEncounter

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import State, StreetAddressLongZip
        from candid.resources.encounter_providers.resources.v_2 import BillingProvider
        from candid.resources.encounters.resources.v_4 import (
            BillableStatusType,
            EncounterSubmissionExpectation,
        )
        from candid.resources.encounters_universal import (
            UniversalEncounterCreateFromPreEncounter,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.encounters.v_4.create_from_pre_encounter_patient_universal(
            request=UniversalEncounterCreateFromPreEncounter(
                submission_expectation=EncounterSubmissionExpectation.TARGET_PROFESSIONAL,
                pre_encounter_patient_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                pre_encounter_appointment_ids=[
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                ],
                billing_provider=BillingProvider(
                    address=StreetAddressLongZip(
                        zip_plus_four_code="zip_plus_four_code",
                        address_1="address1",
                        city="city",
                        state=State.AA,
                        zip_code="zip_code",
                    ),
                    tax_id="tax_id",
                    npi="npi",
                ),
                external_id="external_id",
                patient_authorized_release=True,
                benefits_assigned_to_provider=True,
                provider_accepts_assignment=True,
                billable_status=BillableStatusType.BILLABLE,
            ),
        )
        """
        _response = self._raw_client.create_from_pre_encounter_patient_universal(
            request=request, request_options=request_options
        )
        return _response.data

    def create_from_pre_encounter_patient(
        self, *, request: EncounterCreateFromPreEncounter, request_options: typing.Optional[RequestOptions] = None
    ) -> Encounter:
        """
        Create an encounter from a pre-encounter patient and appointment. This endpoint is intended to be used by consumers who are managing
        patients and appointments in the pre-encounter service and is currently under development. Consumers who are not taking advantage
        of the pre-encounter service should use the standard create endpoint.

        The endpoint will create an encounter from the provided fields, pulling information from the provided patient and appointment objects
        where applicable. In particular, the following fields are populated from the patient and appointment objects:
          - Patient
          - Referring Provider
          - Subscriber Primary
          - Subscriber Secondary
          - Referral Number
          - Responsible Party
          - Guarantor

        Utilizing this endpoint opts you into automatic updating of the encounter when the patient or appointment is updated, assuming the
        encounter has not already been submitted or adjudicated.

        Parameters
        ----------
        request : EncounterCreateFromPreEncounter

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            State,
            StreetAddressLongZip,
        )
        from candid.resources.diagnoses import DiagnosisCreate, DiagnosisTypeCode
        from candid.resources.encounter_providers.resources.v_2 import (
            BillingProvider,
            RenderingProvider,
        )
        from candid.resources.encounters.resources.v_4 import (
            BillableStatusType,
            EncounterCreateFromPreEncounter,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.encounters.v_4.create_from_pre_encounter_patient(
            request=EncounterCreateFromPreEncounter(
                rendering_provider=RenderingProvider(
                    npi="npi",
                ),
                place_of_service_code=FacilityTypeCode.PHARMACY,
                diagnoses=[
                    DiagnosisCreate(
                        code_type=DiagnosisTypeCode.ABF,
                        code="code",
                    ),
                    DiagnosisCreate(
                        code_type=DiagnosisTypeCode.ABF,
                        code="code",
                    ),
                ],
                pre_encounter_patient_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                pre_encounter_appointment_ids=[
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                ],
                billing_provider=BillingProvider(
                    address=StreetAddressLongZip(
                        zip_plus_four_code="zip_plus_four_code",
                        address_1="address1",
                        city="city",
                        state=State.AA,
                        zip_code="zip_code",
                    ),
                    tax_id="tax_id",
                    npi="npi",
                ),
                external_id="external_id",
                patient_authorized_release=True,
                benefits_assigned_to_provider=True,
                provider_accepts_assignment=True,
                billable_status=BillableStatusType.BILLABLE,
            ),
        )
        """
        _response = self._raw_client.create_from_pre_encounter_patient(request=request, request_options=request_options)
        return _response.data

    def update_universal(
        self,
        encounter_id: EncounterId,
        *,
        request: UniversalEncounterUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Encounter:
        """
        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        encounter_id : EncounterId

        request : UniversalEncounterUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.encounters_universal import UniversalEncounterUpdate

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.encounters.v_4.update_universal(
            encounter_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=UniversalEncounterUpdate(),
        )
        """
        _response = self._raw_client.update_universal(encounter_id, request=request, request_options=request_options)
        return _response.data

    def update(
        self,
        encounter_id: EncounterId,
        *,
        request: EncounterUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Encounter:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request : EncounterUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.encounters.resources.v_4 import EncounterUpdate

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.encounters.v_4.update(
            encounter_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=EncounterUpdate(),
        )
        """
        _response = self._raw_client.update(encounter_id, request=request, request_options=request_options)
        return _response.data


class AsyncV4Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV4Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV4Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV4Client
        """
        return self._raw_client

    async def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        sort: typing.Optional[EncounterSortOptions] = None,
        page_token: typing.Optional[PageToken] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        primary_payer_names: typing.Optional[str] = None,
        search_term: typing.Optional[str] = None,
        external_id: typing.Optional[EncounterExternalId] = None,
        diagnoses_updated_since: typing.Optional[dt.datetime] = None,
        tag_ids: typing.Optional[typing.Union[TagId, typing.Sequence[TagId]]] = None,
        work_queue_id: typing.Optional[WorkQueueId] = None,
        billable_status: typing.Optional[BillableStatusType] = None,
        responsible_party: typing.Optional[ResponsiblePartyType] = None,
        owner_of_next_action: typing.Optional[EncounterOwnerOfNextActionType] = None,
        patient_external_id: typing.Optional[str] = None,
        include_merged_patient_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EncounterPage:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        claim_status : typing.Optional[ClaimStatus]
            Indicates the current status of an insurance claim within the billing process.

        sort : typing.Optional[EncounterSortOptions]
            Defaults to created_at:desc.

        page_token : typing.Optional[PageToken]

        date_of_service_min : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.

        date_of_service_max : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.

        primary_payer_names : typing.Optional[str]
            Comma delimited string.

        search_term : typing.Optional[str]
            Filter by any of the following fields: encounter_id, claim_id, patient external_id,
            patient date of birth, patient first name, patient last name,
            or encounter external id.

        external_id : typing.Optional[EncounterExternalId]
            Filter to an exact match on encounter external_id, if one exists.

        diagnoses_updated_since : typing.Optional[dt.datetime]
            ISO 8601 timestamp; ideally in UTC (although not required): 2019-08-24T14:15:22Z.

        tag_ids : typing.Optional[typing.Union[TagId, typing.Sequence[TagId]]]
            Filter by name of tags on encounters.

        work_queue_id : typing.Optional[WorkQueueId]

        billable_status : typing.Optional[BillableStatusType]
            Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.

        responsible_party : typing.Optional[ResponsiblePartyType]
            Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.

        owner_of_next_action : typing.Optional[EncounterOwnerOfNextActionType]
            The party who is responsible for taking the next action on an Encounter, as defined by ownership of open Tasks.

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        include_merged_patient_data : typing.Optional[bool]
            If true and patient_external_id is set, then also include the encounters of all alternative patients.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EncounterPage

        Examples
        --------
        import asyncio
        import datetime

        from candid import AsyncCandidApiClient
        from candid.resources.claims import ClaimStatus
        from candid.resources.encounters.resources.v_4 import EncounterSortOptions

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.encounters.v_4.get_all(
                limit=100,
                claim_status=ClaimStatus.BILLER_RECEIVED,
                sort=EncounterSortOptions.CREATED_AT_ASC,
                page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
                date_of_service_min=datetime.date.fromisoformat(
                    "2019-08-24",
                ),
                date_of_service_max=datetime.date.fromisoformat(
                    "2019-08-25",
                ),
                primary_payer_names="Medicare,Medicaid",
                search_term="doe",
                external_id="123456",
                diagnoses_updated_since=datetime.datetime.fromisoformat(
                    "2019-08-24 14:15:22+00:00",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_all(
            limit=limit,
            claim_status=claim_status,
            sort=sort,
            page_token=page_token,
            date_of_service_min=date_of_service_min,
            date_of_service_max=date_of_service_max,
            primary_payer_names=primary_payer_names,
            search_term=search_term,
            external_id=external_id,
            diagnoses_updated_since=diagnoses_updated_since,
            tag_ids=tag_ids,
            work_queue_id=work_queue_id,
            billable_status=billable_status,
            responsible_party=responsible_party,
            owner_of_next_action=owner_of_next_action,
            patient_external_id=patient_external_id,
            include_merged_patient_data=include_merged_patient_data,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, encounter_id: EncounterId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Encounter:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.encounters.v_4.get(
                encounter_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(encounter_id, request_options=request_options)
        return _response.data

    async def create_universal(
        self, *, request: UniversalEncounterCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> Encounter:
        """
        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        request : UniversalEncounterCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import asyncio
        import datetime

        from candid import AsyncCandidApiClient
        from candid.resources.commons import (
            State,
            StreetAddressLongZip,
            StreetAddressShortZip,
        )
        from candid.resources.encounter_providers.resources.v_2 import BillingProvider
        from candid.resources.encounters.resources.v_4 import (
            BillableStatusType,
            EncounterSubmissionExpectation,
            ResponsiblePartyType,
        )
        from candid.resources.encounters_universal import UniversalEncounterCreate
        from candid.resources.individual import Gender, PatientCreate

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.encounters.v_4.create_universal(
                request=UniversalEncounterCreate(
                    billing_provider=BillingProvider(
                        address=StreetAddressLongZip(
                            zip_plus_four_code="zip_plus_four_code",
                            address_1="address1",
                            city="city",
                            state=State.AA,
                            zip_code="zip_code",
                        ),
                        tax_id="tax_id",
                        npi="npi",
                    ),
                    submission_expectation=EncounterSubmissionExpectation.TARGET_PROFESSIONAL,
                    patient=PatientCreate(
                        external_id="external_id",
                        date_of_birth=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                        address=StreetAddressShortZip(
                            address_1="address1",
                            city="city",
                            state=State.AA,
                            zip_code="zip_code",
                        ),
                        first_name="first_name",
                        last_name="last_name",
                        gender=Gender.MALE,
                    ),
                    responsible_party=ResponsiblePartyType.INSURANCE_PAY,
                    external_id="external_id",
                    patient_authorized_release=True,
                    benefits_assigned_to_provider=True,
                    provider_accepts_assignment=True,
                    billable_status=BillableStatusType.BILLABLE,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_universal(request=request, request_options=request_options)
        return _response.data

    async def create(
        self, *, request: EncounterCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> Encounter:
        """
        Parameters
        ----------
        request : EncounterCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import asyncio
        import datetime

        from candid import AsyncCandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            State,
            StreetAddressLongZip,
            StreetAddressShortZip,
        )
        from candid.resources.diagnoses import DiagnosisCreate, DiagnosisTypeCode
        from candid.resources.encounter_providers.resources.v_2 import (
            BillingProvider,
            RenderingProvider,
        )
        from candid.resources.encounters.resources.v_4 import (
            BillableStatusType,
            EncounterCreate,
            ResponsiblePartyType,
        )
        from candid.resources.individual import Gender, PatientCreate

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.encounters.v_4.create(
                request=EncounterCreate(
                    billing_provider=BillingProvider(
                        address=StreetAddressLongZip(
                            zip_plus_four_code="zip_plus_four_code",
                            address_1="address1",
                            city="city",
                            state=State.AA,
                            zip_code="zip_code",
                        ),
                        tax_id="tax_id",
                        npi="npi",
                    ),
                    diagnoses=[
                        DiagnosisCreate(
                            code_type=DiagnosisTypeCode.ABF,
                            code="code",
                        ),
                        DiagnosisCreate(
                            code_type=DiagnosisTypeCode.ABF,
                            code="code",
                        ),
                    ],
                    place_of_service_code=FacilityTypeCode.PHARMACY,
                    rendering_provider=RenderingProvider(
                        npi="npi",
                    ),
                    patient=PatientCreate(
                        external_id="external_id",
                        date_of_birth=datetime.date.fromisoformat(
                            "2023-01-15",
                        ),
                        address=StreetAddressShortZip(
                            address_1="address1",
                            city="city",
                            state=State.AA,
                            zip_code="zip_code",
                        ),
                        first_name="first_name",
                        last_name="last_name",
                        gender=Gender.MALE,
                    ),
                    responsible_party=ResponsiblePartyType.INSURANCE_PAY,
                    external_id="external_id",
                    patient_authorized_release=True,
                    benefits_assigned_to_provider=True,
                    provider_accepts_assignment=True,
                    billable_status=BillableStatusType.BILLABLE,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def create_from_pre_encounter_patient_universal(
        self,
        *,
        request: UniversalEncounterCreateFromPreEncounter,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Encounter:
        """
        Create an encounter from a pre-encounter patient and appointment. This endpoint is intended to be used by consumers who are managing
        patients and appointments in the pre-encounter service and is currently under development. Consumers who are not taking advantage
        of the pre-encounter service should use the standard create endpoint.

        The endpoint will create an encounter from the provided fields, pulling information from the provided patient and appointment objects
        where applicable. In particular, the following fields are populated from the patient and appointment objects:
          - Patient
          - Referring Provider
          - Subscriber Primary
          - Subscriber Secondary
          - Referral Number
          - Responsible Party
          - Guarantor

        Utilizing this endpoint opts you into automatic updating of the encounter when the patient or appointment is updated, assuming the
        encounter has not already been submitted or adjudicated.

        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        request : UniversalEncounterCreateFromPreEncounter

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import State, StreetAddressLongZip
        from candid.resources.encounter_providers.resources.v_2 import BillingProvider
        from candid.resources.encounters.resources.v_4 import (
            BillableStatusType,
            EncounterSubmissionExpectation,
        )
        from candid.resources.encounters_universal import (
            UniversalEncounterCreateFromPreEncounter,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.encounters.v_4.create_from_pre_encounter_patient_universal(
                request=UniversalEncounterCreateFromPreEncounter(
                    submission_expectation=EncounterSubmissionExpectation.TARGET_PROFESSIONAL,
                    pre_encounter_patient_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    pre_encounter_appointment_ids=[
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                    ],
                    billing_provider=BillingProvider(
                        address=StreetAddressLongZip(
                            zip_plus_four_code="zip_plus_four_code",
                            address_1="address1",
                            city="city",
                            state=State.AA,
                            zip_code="zip_code",
                        ),
                        tax_id="tax_id",
                        npi="npi",
                    ),
                    external_id="external_id",
                    patient_authorized_release=True,
                    benefits_assigned_to_provider=True,
                    provider_accepts_assignment=True,
                    billable_status=BillableStatusType.BILLABLE,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_from_pre_encounter_patient_universal(
            request=request, request_options=request_options
        )
        return _response.data

    async def create_from_pre_encounter_patient(
        self, *, request: EncounterCreateFromPreEncounter, request_options: typing.Optional[RequestOptions] = None
    ) -> Encounter:
        """
        Create an encounter from a pre-encounter patient and appointment. This endpoint is intended to be used by consumers who are managing
        patients and appointments in the pre-encounter service and is currently under development. Consumers who are not taking advantage
        of the pre-encounter service should use the standard create endpoint.

        The endpoint will create an encounter from the provided fields, pulling information from the provided patient and appointment objects
        where applicable. In particular, the following fields are populated from the patient and appointment objects:
          - Patient
          - Referring Provider
          - Subscriber Primary
          - Subscriber Secondary
          - Referral Number
          - Responsible Party
          - Guarantor

        Utilizing this endpoint opts you into automatic updating of the encounter when the patient or appointment is updated, assuming the
        encounter has not already been submitted or adjudicated.

        Parameters
        ----------
        request : EncounterCreateFromPreEncounter

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            State,
            StreetAddressLongZip,
        )
        from candid.resources.diagnoses import DiagnosisCreate, DiagnosisTypeCode
        from candid.resources.encounter_providers.resources.v_2 import (
            BillingProvider,
            RenderingProvider,
        )
        from candid.resources.encounters.resources.v_4 import (
            BillableStatusType,
            EncounterCreateFromPreEncounter,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.encounters.v_4.create_from_pre_encounter_patient(
                request=EncounterCreateFromPreEncounter(
                    rendering_provider=RenderingProvider(
                        npi="npi",
                    ),
                    place_of_service_code=FacilityTypeCode.PHARMACY,
                    diagnoses=[
                        DiagnosisCreate(
                            code_type=DiagnosisTypeCode.ABF,
                            code="code",
                        ),
                        DiagnosisCreate(
                            code_type=DiagnosisTypeCode.ABF,
                            code="code",
                        ),
                    ],
                    pre_encounter_patient_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    pre_encounter_appointment_ids=[
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                    ],
                    billing_provider=BillingProvider(
                        address=StreetAddressLongZip(
                            zip_plus_four_code="zip_plus_four_code",
                            address_1="address1",
                            city="city",
                            state=State.AA,
                            zip_code="zip_code",
                        ),
                        tax_id="tax_id",
                        npi="npi",
                    ),
                    external_id="external_id",
                    patient_authorized_release=True,
                    benefits_assigned_to_provider=True,
                    provider_accepts_assignment=True,
                    billable_status=BillableStatusType.BILLABLE,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_from_pre_encounter_patient(
            request=request, request_options=request_options
        )
        return _response.data

    async def update_universal(
        self,
        encounter_id: EncounterId,
        *,
        request: UniversalEncounterUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Encounter:
        """
        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        encounter_id : EncounterId

        request : UniversalEncounterUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.encounters_universal import UniversalEncounterUpdate

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.encounters.v_4.update_universal(
                encounter_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=UniversalEncounterUpdate(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_universal(
            encounter_id, request=request, request_options=request_options
        )
        return _response.data

    async def update(
        self,
        encounter_id: EncounterId,
        *,
        request: EncounterUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Encounter:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request : EncounterUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Encounter

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.encounters.resources.v_4 import EncounterUpdate

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.encounters.v_4.update(
                encounter_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=EncounterUpdate(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(encounter_id, request=request, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import uuid

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....charge_capture.resources.v_1.types.charge_capture_status import ChargeCaptureStatus
from ....claims.types.claim_status import ClaimStatus
from ....commons.types.charge_capture_claim_creation_id import ChargeCaptureClaimCreationId
from ....commons.types.encounter_id import EncounterId
from ....commons.types.page_token import PageToken
from ....commons.types.sort_direction import SortDirection
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.charge_capture_claim_creation import ChargeCaptureClaimCreation
from .types.charge_capture_claim_creation_page import ChargeCaptureClaimCreationPage
from .types.charge_capture_claim_creation_sort_field import ChargeCaptureClaimCreationSortField
from .types.charge_capture_claim_creation_status import ChargeCaptureClaimCreationStatus
from .types.charge_capture_claim_creation_summary import ChargeCaptureClaimCreationSummary

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def get(
        self,
        charge_capture_claim_creation_id: ChargeCaptureClaimCreationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCaptureClaimCreation:
        """
        Parameters
        ----------
        charge_capture_claim_creation_id : ChargeCaptureClaimCreationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCaptureClaimCreation

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.charge_capture_bundles.v_1.get(
            charge_capture_claim_creation_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get(charge_capture_claim_creation_id, request_options=request_options)
        return _response.data

    def get_summary(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ChargeCaptureClaimCreationSummary:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCaptureClaimCreationSummary

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.charge_capture_bundles.v_1.get_summary()
        """
        _response = self._raw_client.get_summary(request_options=request_options)
        return _response.data

    def resolve_charge_creation_error(
        self,
        charge_capture_bundle_error_id: uuid.UUID,
        *,
        resolved_by: typing.Optional[str] = OMIT,
        resolution_reason: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        charge_capture_bundle_error_id : uuid.UUID

        resolved_by : typing.Optional[str]
            A string, denoting who resolved the error for audit trail purposes.

        resolution_reason : typing.Optional[str]
            A string denoting why or how the error was dealt with for audit trail purposes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.charge_capture_bundles.v_1.resolve_charge_creation_error(
            charge_capture_bundle_error_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.resolve_charge_creation_error(
            charge_capture_bundle_error_id,
            resolved_by=resolved_by,
            resolution_reason=resolution_reason,
            request_options=request_options,
        )
        return _response.data

    def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        sort: typing.Optional[ChargeCaptureClaimCreationSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        patient_external_id: typing.Optional[str] = None,
        claim_creation_status: typing.Optional[ChargeCaptureClaimCreationStatus] = None,
        charge_status: typing.Optional[ChargeCaptureStatus] = None,
        charge_external_id: typing.Optional[str] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        claim_ids: typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]] = None,
        claim_creation_ids: typing.Optional[
            typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]
        ] = None,
        billing_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_facility_name: typing.Optional[str] = None,
        primary_payer_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        claim_creation_category: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tags: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        primary_payer_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        patient_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        has_charge_capture_updates: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCaptureClaimCreationPage:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        sort : typing.Optional[ChargeCaptureClaimCreationSortField]
            Defaults to created_at

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        claim_creation_status : typing.Optional[ChargeCaptureClaimCreationStatus]
            the status of the charge capture Claim Creation, refers to whether it was able to create a claim.

        charge_status : typing.Optional[ChargeCaptureStatus]
            the status of the charge captures

        charge_external_id : typing.Optional[str]
            A client-specified unique ID to associate with this encounter;
            for example, your internal encounter ID or a Dr. Chrono encounter ID.
            This field should not contain PHI.

        date_of_service_min : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        date_of_service_max : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        claim_ids : typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]]
            A list of claim IDs to filter by. This will return all charge capture claim_creations that have a resulting claim with one of the IDs in this list.

        claim_creation_ids : typing.Optional[typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]]
            A list of Claim Creation IDs to filter by.

        billing_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of billing provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        service_facility_name : typing.Optional[str]
            A string to filter by. This will return all charge capture claim_creations which include one or more charges with this service facility name.

        primary_payer_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer IDs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the primary payer IDs in this list.

        rendering_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        rendering_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider names to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        supervising_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        supervising_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider names to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        claim_status : typing.Optional[ClaimStatus]
            the status of the claim to filter by created from charge capture bundle.

        claim_creation_category : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of claim creation categories to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        tags : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of tags to filter by. This will return all charge captures with one of the tags.

        primary_payer_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer names to filter by. This will return all charge captures with one of the names.

        patient_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of patient names to filter by. This will return all charge captures with one of the names.

        has_charge_capture_updates : typing.Optional[bool]
            If true, only return claim_creations that have charge captures that have been updated since the Claim Creation has had a status of BILLED. See the updates property on ChargeCapture for more details.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCaptureClaimCreationPage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.charge_capture_bundles.v_1.get_all()
        """
        _response = self._raw_client.get_all(
            limit=limit,
            sort=sort,
            sort_direction=sort_direction,
            page_token=page_token,
            patient_external_id=patient_external_id,
            claim_creation_status=claim_creation_status,
            charge_status=charge_status,
            charge_external_id=charge_external_id,
            date_of_service_min=date_of_service_min,
            date_of_service_max=date_of_service_max,
            claim_ids=claim_ids,
            claim_creation_ids=claim_creation_ids,
            billing_provider_npis=billing_provider_npis,
            service_facility_name=service_facility_name,
            primary_payer_ids=primary_payer_ids,
            rendering_provider_npis=rendering_provider_npis,
            rendering_provider_names=rendering_provider_names,
            supervising_provider_npis=supervising_provider_npis,
            supervising_provider_names=supervising_provider_names,
            claim_status=claim_status,
            claim_creation_category=claim_creation_category,
            tags=tags,
            primary_payer_names=primary_payer_names,
            patient_names=patient_names,
            has_charge_capture_updates=has_charge_capture_updates,
            request_options=request_options,
        )
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def get(
        self,
        charge_capture_claim_creation_id: ChargeCaptureClaimCreationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCaptureClaimCreation:
        """
        Parameters
        ----------
        charge_capture_claim_creation_id : ChargeCaptureClaimCreationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCaptureClaimCreation

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.charge_capture_bundles.v_1.get(
                charge_capture_claim_creation_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(charge_capture_claim_creation_id, request_options=request_options)
        return _response.data

    async def get_summary(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ChargeCaptureClaimCreationSummary:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCaptureClaimCreationSummary

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.charge_capture_bundles.v_1.get_summary()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_summary(request_options=request_options)
        return _response.data

    async def resolve_charge_creation_error(
        self,
        charge_capture_bundle_error_id: uuid.UUID,
        *,
        resolved_by: typing.Optional[str] = OMIT,
        resolution_reason: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        charge_capture_bundle_error_id : uuid.UUID

        resolved_by : typing.Optional[str]
            A string, denoting who resolved the error for audit trail purposes.

        resolution_reason : typing.Optional[str]
            A string denoting why or how the error was dealt with for audit trail purposes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.charge_capture_bundles.v_1.resolve_charge_creation_error(
                charge_capture_bundle_error_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.resolve_charge_creation_error(
            charge_capture_bundle_error_id,
            resolved_by=resolved_by,
            resolution_reason=resolution_reason,
            request_options=request_options,
        )
        return _response.data

    async def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        sort: typing.Optional[ChargeCaptureClaimCreationSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        patient_external_id: typing.Optional[str] = None,
        claim_creation_status: typing.Optional[ChargeCaptureClaimCreationStatus] = None,
        charge_status: typing.Optional[ChargeCaptureStatus] = None,
        charge_external_id: typing.Optional[str] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        claim_ids: typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]] = None,
        claim_creation_ids: typing.Optional[
            typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]
        ] = None,
        billing_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_facility_name: typing.Optional[str] = None,
        primary_payer_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        rendering_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_npis: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        supervising_provider_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        claim_creation_category: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tags: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        primary_payer_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        patient_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        has_charge_capture_updates: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChargeCaptureClaimCreationPage:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        sort : typing.Optional[ChargeCaptureClaimCreationSortField]
            Defaults to created_at

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        claim_creation_status : typing.Optional[ChargeCaptureClaimCreationStatus]
            the status of the charge capture Claim Creation, refers to whether it was able to create a claim.

        charge_status : typing.Optional[ChargeCaptureStatus]
            the status of the charge captures

        charge_external_id : typing.Optional[str]
            A client-specified unique ID to associate with this encounter;
            for example, your internal encounter ID or a Dr. Chrono encounter ID.
            This field should not contain PHI.

        date_of_service_min : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        date_of_service_max : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-24.
            This date must be the local date in the timezone where the service occurred.

        claim_ids : typing.Optional[typing.Union[EncounterId, typing.Sequence[EncounterId]]]
            A list of claim IDs to filter by. This will return all charge capture claim_creations that have a resulting claim with one of the IDs in this list.

        claim_creation_ids : typing.Optional[typing.Union[ChargeCaptureClaimCreationId, typing.Sequence[ChargeCaptureClaimCreationId]]]
            A list of Claim Creation IDs to filter by.

        billing_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of billing provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        service_facility_name : typing.Optional[str]
            A string to filter by. This will return all charge capture claim_creations which include one or more charges with this service facility name.

        primary_payer_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer IDs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the primary payer IDs in this list.

        rendering_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        rendering_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of rendering provider names to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        supervising_provider_npis : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider NPIs to filter by. This will return all charge capture claim_creations which include one or more charges with one of the NPIs in this list.

        supervising_provider_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of supervising provider names to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        claim_status : typing.Optional[ClaimStatus]
            the status of the claim to filter by created from charge capture bundle.

        claim_creation_category : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of claim creation categories to filter by. This will return all charge capture claim_creations which include one or more charges with one of the names in this list.

        tags : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of tags to filter by. This will return all charge captures with one of the tags.

        primary_payer_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of primary payer names to filter by. This will return all charge captures with one of the names.

        patient_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A list of patient names to filter by. This will return all charge captures with one of the names.

        has_charge_capture_updates : typing.Optional[bool]
            If true, only return claim_creations that have charge captures that have been updated since the Claim Creation has had a status of BILLED. See the updates property on ChargeCapture for more details.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargeCaptureClaimCreationPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.charge_capture_bundles.v_1.get_all()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_all(
            limit=limit,
            sort=sort,
            sort_direction=sort_direction,
            page_token=page_token,
            patient_external_id=patient_external_id,
            claim_creation_status=claim_creation_status,
            charge_status=charge_status,
            charge_external_id=charge_external_id,
            date_of_service_min=date_of_service_min,
            date_of_service_max=date_of_service_max,
            claim_ids=claim_ids,
            claim_creation_ids=claim_creation_ids,
            billing_provider_npis=billing_provider_npis,
            service_facility_name=service_facility_name,
            primary_payer_ids=primary_payer_ids,
            rendering_provider_npis=rendering_provider_npis,
            rendering_provider_names=rendering_provider_names,
            supervising_provider_npis=supervising_provider_npis,
            supervising_provider_names=supervising_provider_names,
            claim_status=claim_status,
            claim_creation_category=claim_creation_category,
            tags=tags,
            primary_payer_names=primary_payer_names,
            patient_names=patient_names,
            has_charge_capture_updates=has_charge_capture_updates,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class PaymentMethod_Cash(UniversalBaseModel):
    type: typing.Literal["cash"] = "cash"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethod_Check(UniversalBaseModel):
    type: typing.Literal["check"] = "check"
    check_number: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethod_Card(UniversalBaseModel):
    type: typing.Literal["card"] = "card"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethod_MoneyOrder(UniversalBaseModel):
    type: typing.Literal["money_order"] = "money_order"
    money_order_serial_number: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


PaymentMethod = typing_extensions.Annotated[
    typing.Union[PaymentMethod_Cash, PaymentMethod_Check, PaymentMethod_Card, PaymentMethod_MoneyOrder],
    pydantic.Field(discriminator="type"),
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.invoice_id import InvoiceId
from .....commons.types.organization_id import OrganizationId
from .....commons.types.patient_external_id import PatientExternalId
from .....financials.types.allocation import Allocation
from .....financials.types.patient_transaction_source import PatientTransactionSource
from .patient_payment_id import PatientPaymentId
from .payment_method_detail import PaymentMethodDetail


class PatientPayment(UniversalBaseModel):
    patient_payment_id: PatientPaymentId
    organization_id: OrganizationId
    source_internal_id: typing.Optional[str] = None
    payment_source: PatientTransactionSource
    amount_cents: int
    patient_external_id: PatientExternalId
    payment_timestamp: typing.Optional[dt.datetime] = None
    payment_note: typing.Optional[str] = None
    allocations: typing.List[Allocation]
    invoice: typing.Optional[InvoiceId] = None
    payment_method_detail: typing.Optional[PaymentMethodDetail] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class Clearinghouse(enum.StrEnum):
    CHANGE_HEALTHCARE = "CHANGE_HEALTHCARE"
    OLIVE = "OLIVE"
    UCSF_CIRIUS = "UCSF_CIRIUS"
    AVAILITY = "AVAILITY"
    JOPARI = "JOPARI"
    PAYER_PORTAL = "PAYER_PORTAL"
    STEDI = "STEDI"
    WAYSTAR = "WAYSTAR"
    UNKNOWN = "UNKNOWN"
    _UNKNOWN = "__CLEARINGHOUSE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "Clearinghouse":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        change_healthcare: typing.Callable[[], T_Result],
        olive: typing.Callable[[], T_Result],
        ucsf_cirius: typing.Callable[[], T_Result],
        availity: typing.Callable[[], T_Result],
        jopari: typing.Callable[[], T_Result],
        payer_portal: typing.Callable[[], T_Result],
        stedi: typing.Callable[[], T_Result],
        waystar: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is Clearinghouse.CHANGE_HEALTHCARE:
            return change_healthcare()
        if self is Clearinghouse.OLIVE:
            return olive()
        if self is Clearinghouse.UCSF_CIRIUS:
            return ucsf_cirius()
        if self is Clearinghouse.AVAILITY:
            return availity()
        if self is Clearinghouse.JOPARI:
            return jopari()
        if self is Clearinghouse.PAYER_PORTAL:
            return payer_portal()
        if self is Clearinghouse.STEDI:
            return stedi()
        if self is Clearinghouse.WAYSTAR:
            return waystar()
        if self is Clearinghouse.UNKNOWN:
            return unknown()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from .errors.invalid_content_type_error import InvalidContentTypeError
from .errors.too_many_requests_error import TooManyRequestsError
from .types.auth_get_token_response import AuthGetTokenResponse
from .types.invalid_content_type_error_type import InvalidContentTypeErrorType
from .types.too_many_requests_error_type import TooManyRequestsErrorType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDefaultClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_token(
        self, *, client_id: str, client_secret: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AuthGetTokenResponse]:
        """
        <Callout intent="info">
        Candid Health SDKs automatically handle authentication workflows after configuring them with the `client_id` and
        `client_secret`.
        </Callout>

        Candid Health utilizes the [OAuth 2.0 bearer token authentication scheme](https://developer.mozilla.org/en-US/docs/Web/HTTP/Authentication) in our auth flow. You obtain the bearer token for all
        subsequent API requests via the `/auth/v2/token` endpoint defined below, which requires you to provide your `client_id` and `client_secret`. Your `client_id` and `client_secret` can be [generated](https://support.joincandidhealth.com/hc/en-us/articles/23065219476244--Generating-Candid-API-Keys) from the "Users & Credentials" tab by your org admin.

        The `/auth/v2/token` endpoint accepts both `Content-Type: application/json` and `Content-Type: application/x-www-form-urlencoded`. The request body should contain the `client_id` and `client_secret` as follows:

        ```json
        {
          "client_id": "YOUR_CLIENT_ID",
          "client_secret": "YOUR_CLIENT_SECRET"
        }
        ```
        or as URL-encoded form data:

        ```
        client_id=YOUR_CLIENT_ID&client_secret=YOUR_CLIENT_SECRET
        ```

        The bearer token is a signed [JWT](https://jwt.io/). The public key for the JWT can be found [here](https://candidhealth.auth0.com/pem) for any verification workflows.

        The bearer token should be provided in the `Authorization` header for all subsequent API calls.

        <Callout intent="warning">
        The bearer token expires 5 hours after it has been created. After it has expired, the client will receive an "HTTP 401
        Unauthorized" error, at which point the client should generate a new token. It is important that tokens be reused between
        requests; if the client attempts to generate a token too often, it will be rate-limited and will receive an `HTTP 429 Too Many Requests` error.
        </Callout>

        Parameters
        ----------
        client_id : str
            Your application's Client ID.

        client_secret : str
            Your application's Client Secret.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AuthGetTokenResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/auth/v2/token",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json={
                "client_id": client_id,
                "client_secret": client_secret,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                AuthGetTokenResponse,
                parse_obj_as(
                    type_=AuthGetTokenResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "TooManyRequestsError":
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequestsErrorType,
                        parse_obj_as(
                            type_=TooManyRequestsErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InvalidContentTypeError":
                raise InvalidContentTypeError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidContentTypeErrorType,
                        parse_obj_as(
                            type_=InvalidContentTypeErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDefaultClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_token(
        self, *, client_id: str, client_secret: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AuthGetTokenResponse]:
        """
        <Callout intent="info">
        Candid Health SDKs automatically handle authentication workflows after configuring them with the `client_id` and
        `client_secret`.
        </Callout>

        Candid Health utilizes the [OAuth 2.0 bearer token authentication scheme](https://developer.mozilla.org/en-US/docs/Web/HTTP/Authentication) in our auth flow. You obtain the bearer token for all
        subsequent API requests via the `/auth/v2/token` endpoint defined below, which requires you to provide your `client_id` and `client_secret`. Your `client_id` and `client_secret` can be [generated](https://support.joincandidhealth.com/hc/en-us/articles/23065219476244--Generating-Candid-API-Keys) from the "Users & Credentials" tab by your org admin.

        The `/auth/v2/token` endpoint accepts both `Content-Type: application/json` and `Content-Type: application/x-www-form-urlencoded`. The request body should contain the `client_id` and `client_secret` as follows:

        ```json
        {
          "client_id": "YOUR_CLIENT_ID",
          "client_secret": "YOUR_CLIENT_SECRET"
        }
        ```
        or as URL-encoded form data:

        ```
        client_id=YOUR_CLIENT_ID&client_secret=YOUR_CLIENT_SECRET
        ```

        The bearer token is a signed [JWT](https://jwt.io/). The public key for the JWT can be found [here](https://candidhealth.auth0.com/pem) for any verification workflows.

        The bearer token should be provided in the `Authorization` header for all subsequent API calls.

        <Callout intent="warning">
        The bearer token expires 5 hours after it has been created. After it has expired, the client will receive an "HTTP 401
        Unauthorized" error, at which point the client should generate a new token. It is important that tokens be reused between
        requests; if the client attempts to generate a token too often, it will be rate-limited and will receive an `HTTP 429 Too Many Requests` error.
        </Callout>

        Parameters
        ----------
        client_id : str
            Your application's Client ID.

        client_secret : str
            Your application's Client Secret.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AuthGetTokenResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/auth/v2/token",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json={
                "client_id": client_id,
                "client_secret": client_secret,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                AuthGetTokenResponse,
                parse_obj_as(
                    type_=AuthGetTokenResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "TooManyRequestsError":
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequestsErrorType,
                        parse_obj_as(
                            type_=TooManyRequestsErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InvalidContentTypeError":
                raise InvalidContentTypeError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidContentTypeErrorType,
                        parse_obj_as(
                            type_=InvalidContentTypeErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class OccurrenceSpanCodeQualifier(enum.StrEnum):
    CBI = "BI"
    """
    Occurrence Span
    """

    _UNKNOWN = "__OCCURRENCESPANCODEQUALIFIER_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "OccurrenceSpanCodeQualifier":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(self, cbi: typing.Callable[[], T_Result], _unknown_member: typing.Callable[[str], T_Result]) -> T_Result:
        if self is OccurrenceSpanCodeQualifier.CBI:
            return cbi()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ........core import enum

T_Result = typing.TypeVar("T_Result")


class AppointmentStatus(enum.StrEnum):
    PENDING = "PENDING"
    NOT_READY = "NOT_READY"
    READY = "READY"
    CHECKED_IN = "CHECKED_IN"
    _UNKNOWN = "__APPOINTMENTSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "AppointmentStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        pending: typing.Callable[[], T_Result],
        not_ready: typing.Callable[[], T_Result],
        ready: typing.Callable[[], T_Result],
        checked_in: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is AppointmentStatus.PENDING:
            return pending()
        if self is AppointmentStatus.NOT_READY:
            return not_ready()
        if self is AppointmentStatus.READY:
            return ready()
        if self is AppointmentStatus.CHECKED_IN:
            return checked_in()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class DiagnosisRelatedGroupCodeQualifier(enum.StrEnum):
    CDR = "DR"
    """
    Diagnosis Related Group (DRG)
    """

    _UNKNOWN = "__DIAGNOSISRELATEDGROUPCODEQUALIFIER_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "DiagnosisRelatedGroupCodeQualifier":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(self, cdr: typing.Callable[[], T_Result], _unknown_member: typing.Callable[[str], T_Result]) -> T_Result:
        if self is DiagnosisRelatedGroupCodeQualifier.CDR:
            return cdr()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .measurement_unit_code import MeasurementUnitCode
from .service_id_qualifier import ServiceIdQualifier


class DrugIdentificationOptional(UniversalBaseModel):
    service_id_qualifier: typing.Optional[ServiceIdQualifier] = None
    national_drug_code: typing.Optional[str] = None
    national_drug_unit_count: typing.Optional[str] = None
    measurement_unit_code: typing.Optional[MeasurementUnitCode] = None
    link_sequence_number: typing.Optional[str] = None
    pharmacy_prescription_number: typing.Optional[str] = None
    conversion_formula: typing.Optional[str] = None
    drug_description: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.claim_submission_payer_responsibility_type import ClaimSubmissionPayerResponsibilityType
from .....commons.types.intended_submission_medium import IntendedSubmissionMedium
from .....x_12.resources.v_1.types.type_of_bill_frequency_code import TypeOfBillFrequencyCode


class ClaimSubmissionRecordCreate(UniversalBaseModel):
    """
    Data about each external submission.

    Examples
    --------
    import datetime

    from candid.resources.claim_submission.resources.v_1 import (
        ClaimSubmissionRecordCreate,
    )
    from candid.resources.commons import (
        ClaimSubmissionPayerResponsibilityType,
        IntendedSubmissionMedium,
    )
    from candid.resources.x_12.resources.v_1 import TypeOfBillFrequencyCode

    ClaimSubmissionRecordCreate(
        submitted_at=datetime.datetime.fromisoformat(
            "2023-01-01 13:00:00+00:00",
        ),
        claim_frequency_code=TypeOfBillFrequencyCode.C_1,
        payer_responsibility=ClaimSubmissionPayerResponsibilityType.PRIMARY,
        intended_submission_medium=IntendedSubmissionMedium.ELECTRONIC,
    )
    """

    submitted_at: dt.datetime = pydantic.Field()
    """
    When the claim was submitted to the payer.
    """

    claim_frequency_code: typing.Optional[TypeOfBillFrequencyCode] = None
    payer_responsibility: typing.Optional[ClaimSubmissionPayerResponsibilityType] = None
    intended_submission_medium: typing.Optional[IntendedSubmissionMedium] = pydantic.Field(default=None)
    """
    The medium by which the claim was submitted to the payer: paper or electronic.
    If omitted, defaults to electronic.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.delay_reason_code import DelayReasonCode
from .....commons.types.encounter_external_id import EncounterExternalId
from .....commons.types.facility_type_code import FacilityTypeCode
from .....tags.types.tag_id import TagId
from .billable_status_type import BillableStatusType
from .prior_authorization_number import PriorAuthorizationNumber
from .responsible_party_type import ResponsiblePartyType
from .service_authorization_exception_code import ServiceAuthorizationExceptionCode
from .synchronicity_type import SynchronicityType


class EncounterOptional(UniversalBaseModel):
    benefits_assigned_to_provider: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this patient has authorized insurance payments to be made to you, not them. If false, patient may receive reimbursement. Box 13 on the CMS-1500 claim form or Form Locator 53 on a UB-04 claim form.
    """

    prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = pydantic.Field(default=None)
    """
    Box 23 on the CMS-1500 claim form or Form Locator 63 on a UB-04 claim form.
    """

    external_id: typing.Optional[EncounterExternalId] = pydantic.Field(default=None)
    """
    A client-specified unique ID to associate with this encounter;
    for example, your internal encounter ID or a Dr. Chrono encounter ID.
    This field should not contain PHI.
    """

    date_of_service: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    Date formatted as YYYY-MM-DD; eg: 2019-08-24.
    This date must be the local date in the timezone where the service occurred.
    Box 24a on the CMS-1500 claim form or Form Locator 45 on the UB-04 claim form.
    If service occurred over a range of dates, this should be the start date.
    If service lines have distinct date_of_service values, updating the encounter's date_of_service will fail. If all service line date_of_service values are the same, updating the encounter's date_of_service will update all service line date_of_service values.
    """

    tag_ids: typing.Optional[typing.List[TagId]] = pydantic.Field(default=None)
    """
    Names of tags that should be on the encounter.  Note all tags on encounter will be overridden with this list.
    """

    billable_status: typing.Optional[BillableStatusType] = pydantic.Field(default=None)
    """
    Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.
    """

    responsible_party: typing.Optional[ResponsiblePartyType] = pydantic.Field(default=None)
    """
    Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.
    """

    provider_accepts_assignment: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether you have accepted the patient's authorization for insurance payments to be made to you, not them. Box 27 on the CMS-1500 claim form. There is no exact equivalent of this field on a UB-04 claim, however contributes to the concept of Form Locator 53.
    """

    synchronicity: typing.Optional[SynchronicityType] = pydantic.Field(default=None)
    """
    Whether or not this was a synchronous or asynchronous encounter. Asynchronous encounters occur when providers and patients communicate online using forms, instant messaging, or other pre-recorded digital mediums. Synchronous encounters occur in live, real-time settings where the patient interacts directly with the provider, such as over video or a phone call.
    """

    place_of_service_code: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    Box 24B on the CMS-1500 claim form. 837p Loop2300, CLM-05-1. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    appointment_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Human-readable description of the appointment type (ex: "Acupuncture - Headaches").
    """

    end_date_of_service: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    Date formatted as YYYY-MM-DD; eg: 2019-08-25.
    This date must be the local date in the timezone where the service occurred.
    If omitted, the Encounter is assumed to be for a single day.
    Must not be temporally before the date_of_service field.
    If service lines have distinct end_date_of_service values, updating the encounter's end_date_of_service will fail. If all service line end_date_of_service values are the same, updating the encounter's end_date_of_service will update all service line date_of_service values.
    """

    additional_information: typing.Optional[str] = pydantic.Field(default=None)
    """
    Defines additional information on the claim needed by the payer.
    Box 19 on the CMS-1500 claim form or Form Locator 80 on a UB-04 claim form.
    """

    service_authorization_exception_code: typing.Optional[ServiceAuthorizationExceptionCode] = pydantic.Field(
        default=None
    )
    """
    837p Loop2300 REF*4N
    Required when mandated by government law or regulation to obtain authorization for specific service(s) but, for the
    reasons listed in one of the enum values of ServiceAuthorizationExceptionCode, the service was performed without
    obtaining the authorization.
    """

    admission_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP*435, CMS-1500 Box 18 or UB-04 Form Locator 12.
    Required on all ambulance claims when the patient was known to be admitted to the hospital.
    OR
    Required on all claims involving inpatient medical visits.
    """

    discharge_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP*096, CMS-1500 Box 18 Required for inpatient claims when the patient was discharged from the facility and the discharge date is known. Not used on an institutional claim.
    """

    onset_of_current_illness_or_symptom_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP*431, CMS-1500 Box 14
    Required for the initial medical service or visit performed in response to a medical emergency when the date is available and is different than the date of service.
    OR
    This date is the onset of acute symptoms for the current illness or condition.
     For UB-04 claims, this is populated separately via occurrence codes.
    """

    last_menstrual_period_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP*484, CMS-1500 Box 14
    Required when, in the judgment of the provider, the services on this claim are related to the patient's pregnancy.de
    This field is populated separately via occurrence codes for UB-04 claim forms.
    """

    delay_reason_code: typing.Optional[DelayReasonCode] = pydantic.Field(default=None)
    """
    837i Loop2300, CLM-1300 Box 20
    Code indicating the reason why a request was delayed
    """

    patient_authorized_release: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this patient has authorized the release of medical information
    for billing purpose.
    Box 12 on the CMS-1500 claim form  or Form Locator 52 on a UB-04 claim form.
    """

    referral_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Refers to REF*9F on the 837p. Value cannot be greater than 50 characters.
    """

    secondary_payer_carrier_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    When Medicaid is billed as the secondary payer the Carrier Code is used to identify the primary payer. This is required for certain states.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.email import Email
from .....commons.types.phone_number_optional import PhoneNumberOptional
from .....commons.types.street_address_short_zip_optional import StreetAddressShortZipOptional


class GuarantorOptional(UniversalBaseModel):
    first_name: typing.Optional[str] = None
    last_name: typing.Optional[str] = None
    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for the guarantor assigned by an external system.
    """

    date_of_birth: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    Date formatted as YYYY-MM-DD; eg: 2019-08-25.
    """

    address: typing.Optional[StreetAddressShortZipOptional] = None
    phone_numbers: typing.Optional[typing.List[PhoneNumberOptional]] = None
    phone_consent: typing.Optional[bool] = None
    email: typing.Optional[Email] = None
    email_consent: typing.Optional[bool] = None
    auto_charge_consent: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.api_error import ApiError
from ..types.organization_not_authorized_error_message import OrganizationNotAuthorizedErrorMessage


class OrganizationNotAuthorizedError(ApiError):
    def __init__(
        self, body: OrganizationNotAuthorizedErrorMessage, headers: typing.Optional[typing.Dict[str, str]] = None
    ):
        super().__init__(status_code=400, headers=headers, body=body)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Address,
        BenefitType,
        BenefitsRelatedEntity,
        Coverage,
        CoverageBenefits,
        CoverageDetails,
        CoverageEligibilityCheckResponse,
        CoverageLevel,
        CoverageStatus,
        CoverageValue,
        CoverageValueUnit,
        CoveragesPage,
        ExpandedMemberInfo,
        InsurancePlan,
        InsuranceTypeCode,
        LatestEligibilityCheck,
        MemberInfo,
        MutableCoverage,
        NetworkType,
        PayerPlanGroupFields,
        PlanCoverage,
        PlanCoverageDetails,
        PlanDate,
        PlanMetadata,
        RelatedEntityContact,
        ServiceCoverage,
        ServiceCoverageDetails,
        ServiceTypeCode,
        Subscriber,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "Address": ".types",
    "BenefitType": ".types",
    "BenefitsRelatedEntity": ".types",
    "Coverage": ".types",
    "CoverageBenefits": ".types",
    "CoverageDetails": ".types",
    "CoverageEligibilityCheckResponse": ".types",
    "CoverageLevel": ".types",
    "CoverageStatus": ".types",
    "CoverageValue": ".types",
    "CoverageValueUnit": ".types",
    "CoveragesPage": ".types",
    "ExpandedMemberInfo": ".types",
    "InsurancePlan": ".types",
    "InsuranceTypeCode": ".types",
    "LatestEligibilityCheck": ".types",
    "MemberInfo": ".types",
    "MutableCoverage": ".types",
    "NetworkType": ".types",
    "PayerPlanGroupFields": ".types",
    "PlanCoverage": ".types",
    "PlanCoverageDetails": ".types",
    "PlanDate": ".types",
    "PlanMetadata": ".types",
    "RelatedEntityContact": ".types",
    "ServiceCoverage": ".types",
    "ServiceCoverageDetails": ".types",
    "ServiceTypeCode": ".types",
    "Subscriber": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Address",
    "BenefitType",
    "BenefitsRelatedEntity",
    "Coverage",
    "CoverageBenefits",
    "CoverageDetails",
    "CoverageEligibilityCheckResponse",
    "CoverageLevel",
    "CoverageStatus",
    "CoverageValue",
    "CoverageValueUnit",
    "CoveragesPage",
    "ExpandedMemberInfo",
    "InsurancePlan",
    "InsuranceTypeCode",
    "LatestEligibilityCheck",
    "MemberInfo",
    "MutableCoverage",
    "NetworkType",
    "PayerPlanGroupFields",
    "PlanCoverage",
    "PlanCoverageDetails",
    "PlanDate",
    "PlanMetadata",
    "RelatedEntityContact",
    "ServiceCoverage",
    "ServiceCoverageDetails",
    "ServiceTypeCode",
    "Subscriber",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawPreEncounterClient, RawPreEncounterClient

if typing.TYPE_CHECKING:
    from .resources.appointments.client import AppointmentsClient, AsyncAppointmentsClient
    from .resources.coverages.client import AsyncCoveragesClient, CoveragesClient
    from .resources.eligibility_checks.client import AsyncEligibilityChecksClient, EligibilityChecksClient
    from .resources.images.client import AsyncImagesClient, ImagesClient
    from .resources.lists.client import AsyncListsClient, ListsClient
    from .resources.notes.client import AsyncNotesClient, NotesClient
    from .resources.patients.client import AsyncPatientsClient, PatientsClient
    from .resources.tags.client import AsyncTagsClient, TagsClient


class PreEncounterClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPreEncounterClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._appointments: typing.Optional[AppointmentsClient] = None
        self._coverages: typing.Optional[CoveragesClient] = None
        self._eligibility_checks: typing.Optional[EligibilityChecksClient] = None
        self._images: typing.Optional[ImagesClient] = None
        self._lists: typing.Optional[ListsClient] = None
        self._notes: typing.Optional[NotesClient] = None
        self._patients: typing.Optional[PatientsClient] = None
        self._tags: typing.Optional[TagsClient] = None

    @property
    def with_raw_response(self) -> RawPreEncounterClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPreEncounterClient
        """
        return self._raw_client

    @property
    def appointments(self):
        if self._appointments is None:
            from .resources.appointments.client import AppointmentsClient  # noqa: E402

            self._appointments = AppointmentsClient(client_wrapper=self._client_wrapper)
        return self._appointments

    @property
    def coverages(self):
        if self._coverages is None:
            from .resources.coverages.client import CoveragesClient  # noqa: E402

            self._coverages = CoveragesClient(client_wrapper=self._client_wrapper)
        return self._coverages

    @property
    def eligibility_checks(self):
        if self._eligibility_checks is None:
            from .resources.eligibility_checks.client import EligibilityChecksClient  # noqa: E402

            self._eligibility_checks = EligibilityChecksClient(client_wrapper=self._client_wrapper)
        return self._eligibility_checks

    @property
    def images(self):
        if self._images is None:
            from .resources.images.client import ImagesClient  # noqa: E402

            self._images = ImagesClient(client_wrapper=self._client_wrapper)
        return self._images

    @property
    def lists(self):
        if self._lists is None:
            from .resources.lists.client import ListsClient  # noqa: E402

            self._lists = ListsClient(client_wrapper=self._client_wrapper)
        return self._lists

    @property
    def notes(self):
        if self._notes is None:
            from .resources.notes.client import NotesClient  # noqa: E402

            self._notes = NotesClient(client_wrapper=self._client_wrapper)
        return self._notes

    @property
    def patients(self):
        if self._patients is None:
            from .resources.patients.client import PatientsClient  # noqa: E402

            self._patients = PatientsClient(client_wrapper=self._client_wrapper)
        return self._patients

    @property
    def tags(self):
        if self._tags is None:
            from .resources.tags.client import TagsClient  # noqa: E402

            self._tags = TagsClient(client_wrapper=self._client_wrapper)
        return self._tags


class AsyncPreEncounterClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPreEncounterClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._appointments: typing.Optional[AsyncAppointmentsClient] = None
        self._coverages: typing.Optional[AsyncCoveragesClient] = None
        self._eligibility_checks: typing.Optional[AsyncEligibilityChecksClient] = None
        self._images: typing.Optional[AsyncImagesClient] = None
        self._lists: typing.Optional[AsyncListsClient] = None
        self._notes: typing.Optional[AsyncNotesClient] = None
        self._patients: typing.Optional[AsyncPatientsClient] = None
        self._tags: typing.Optional[AsyncTagsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawPreEncounterClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPreEncounterClient
        """
        return self._raw_client

    @property
    def appointments(self):
        if self._appointments is None:
            from .resources.appointments.client import AsyncAppointmentsClient  # noqa: E402

            self._appointments = AsyncAppointmentsClient(client_wrapper=self._client_wrapper)
        return self._appointments

    @property
    def coverages(self):
        if self._coverages is None:
            from .resources.coverages.client import AsyncCoveragesClient  # noqa: E402

            self._coverages = AsyncCoveragesClient(client_wrapper=self._client_wrapper)
        return self._coverages

    @property
    def eligibility_checks(self):
        if self._eligibility_checks is None:
            from .resources.eligibility_checks.client import AsyncEligibilityChecksClient  # noqa: E402

            self._eligibility_checks = AsyncEligibilityChecksClient(client_wrapper=self._client_wrapper)
        return self._eligibility_checks

    @property
    def images(self):
        if self._images is None:
            from .resources.images.client import AsyncImagesClient  # noqa: E402

            self._images = AsyncImagesClient(client_wrapper=self._client_wrapper)
        return self._images

    @property
    def lists(self):
        if self._lists is None:
            from .resources.lists.client import AsyncListsClient  # noqa: E402

            self._lists = AsyncListsClient(client_wrapper=self._client_wrapper)
        return self._lists

    @property
    def notes(self):
        if self._notes is None:
            from .resources.notes.client import AsyncNotesClient  # noqa: E402

            self._notes = AsyncNotesClient(client_wrapper=self._client_wrapper)
        return self._notes

    @property
    def patients(self):
        if self._patients is None:
            from .resources.patients.client import AsyncPatientsClient  # noqa: E402

            self._patients = AsyncPatientsClient(client_wrapper=self._client_wrapper)
        return self._patients

    @property
    def tags(self):
        if self._tags is None:
            from .resources.tags.client import AsyncTagsClient  # noqa: E402

            self._tags = AsyncTagsClient(client_wrapper=self._client_wrapper)
        return self._tags



# This file was auto-generated by Fern from our API Definition.

import typing
import uuid
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.sort_direction import SortDirection
from ....commons.types.state import State
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from ..v_2.errors.contract_invalid_expiration_date_http_error import ContractInvalidExpirationDateHttpError
from ..v_2.errors.contract_is_linked_to_fee_schedule_http_error import ContractIsLinkedToFeeScheduleHttpError
from ..v_2.types.contract_invalid_expiration_date_error import ContractInvalidExpirationDateError
from ..v_2.types.contract_is_linked_to_fee_schedule_error import ContractIsLinkedToFeeScheduleError
from ..v_2.types.contract_sort_field import ContractSortField
from ..v_2.types.contract_status import ContractStatus
from .types.contract_create_union import ContractCreateUnion
from .types.contract_id import ContractId
from .types.contract_service_facility import ContractServiceFacility
from .types.contract_service_facility_id import ContractServiceFacilityId
from .types.contract_type import ContractType
from .types.contract_update_union import ContractUpdateUnion
from .types.contract_with_providers_union import ContractWithProvidersUnion
from .types.contracting_provider_id import ContractingProviderId
from .types.contracts_page import ContractsPage
from .types.rendering_providerid import RenderingProviderid

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, contract_id: ContractId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ContractWithProvidersUnion]:
        """
        Parameters
        ----------
        contract_id : ContractId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ContractWithProvidersUnion]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/contracts/v3/{jsonable_encoder(contract_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractWithProvidersUnion,
                parse_obj_as(
                    type_=ContractWithProvidersUnion,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        type: typing.Optional[ContractType] = None,
        contracting_provider_id: typing.Optional[ContractingProviderId] = None,
        rendering_provider_ids: typing.Optional[
            typing.Union[RenderingProviderid, typing.Sequence[RenderingProviderid]]
        ] = None,
        payer_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        states: typing.Optional[typing.Union[State, typing.Sequence[State]]] = None,
        contract_status: typing.Optional[ContractStatus] = None,
        sort: typing.Optional[ContractSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ContractsPage]:
        """
        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Max number of contracts returned. Defaults to 1000. Max is 1000.

        type : typing.Optional[ContractType]
            The type of contract

        contracting_provider_id : typing.Optional[ContractingProviderId]

        rendering_provider_ids : typing.Optional[typing.Union[RenderingProviderid, typing.Sequence[RenderingProviderid]]]

        payer_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter to contracts that include any of the included payer names.

        states : typing.Optional[typing.Union[State, typing.Sequence[State]]]

        contract_status : typing.Optional[ContractStatus]
            The status of the contract. Defaults to `pending`

        sort : typing.Optional[ContractSortField]
            Potentially sort by a contract related attribute.  Defaults to created_at

        sort_direction : typing.Optional[SortDirection]
            Direction of sort, defaulting to desc

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ContractsPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/contracts/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "page_token": page_token,
                "limit": limit,
                "type": type,
                "contracting_provider_id": contracting_provider_id,
                "rendering_provider_ids": rendering_provider_ids,
                "payer_names": payer_names,
                "states": states,
                "contract_status": contract_status,
                "sort": sort,
                "sort_direction": sort_direction,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractsPage,
                parse_obj_as(
                    type_=ContractsPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, request: ContractCreateUnion, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ContractWithProvidersUnion]:
        """
        Creates a new contract within the user's current organization

        Parameters
        ----------
        request : ContractCreateUnion

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ContractWithProvidersUnion]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/contracts/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractWithProvidersUnion,
                parse_obj_as(
                    type_=ContractWithProvidersUnion,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, contract_id: ContractId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Parameters
        ----------
        contract_id : ContractId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/contracts/v3/{jsonable_encoder(contract_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "ContractIsLinkedToFeeScheduleHttpError":
                raise ContractIsLinkedToFeeScheduleHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ContractIsLinkedToFeeScheduleError,
                        parse_obj_as(
                            type_=ContractIsLinkedToFeeScheduleError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        contract_id: ContractId,
        *,
        request: ContractUpdateUnion,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ContractWithProvidersUnion]:
        """
        Parameters
        ----------
        contract_id : ContractId

        request : ContractUpdateUnion

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ContractWithProvidersUnion]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/contracts/v3/{jsonable_encoder(contract_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractWithProvidersUnion,
                parse_obj_as(
                    type_=ContractWithProvidersUnion,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "ContractInvalidExpirationDateHttpError":
                raise ContractInvalidExpirationDateHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ContractInvalidExpirationDateError,
                        parse_obj_as(
                            type_=ContractInvalidExpirationDateError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_contract_service_facility(
        self,
        contract_id: ContractId,
        *,
        service_facility_id: uuid.UUID,
        provider_ids: typing.Set[uuid.UUID],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ContractServiceFacility]:
        """
        Parameters
        ----------
        contract_id : ContractId

        service_facility_id : uuid.UUID
            The UUID of the service facility

        provider_ids : typing.Set[uuid.UUID]
            The providers who are authorized under the contract

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ContractServiceFacility]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/contracts/v3/{jsonable_encoder(contract_id)}/service-facilities",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json={
                "service_facility_id": service_facility_id,
                "provider_ids": provider_ids,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractServiceFacility,
                parse_obj_as(
                    type_=ContractServiceFacility,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_contract_service_facility(
        self,
        contract_id: ContractId,
        contract_service_facility_id: ContractServiceFacilityId,
        *,
        service_facility_id: typing.Optional[uuid.UUID] = OMIT,
        provider_ids: typing.Optional[typing.Set[uuid.UUID]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ContractServiceFacility]:
        """
        Parameters
        ----------
        contract_id : ContractId

        contract_service_facility_id : ContractServiceFacilityId

        service_facility_id : typing.Optional[uuid.UUID]
            The UUID of the service facility

        provider_ids : typing.Optional[typing.Set[uuid.UUID]]
            The providers who are authorized under the contract

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ContractServiceFacility]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/contracts/v3/{jsonable_encoder(contract_id)}/service-facilities/{jsonable_encoder(contract_service_facility_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "service_facility_id": service_facility_id,
                "provider_ids": provider_ids,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractServiceFacility,
                parse_obj_as(
                    type_=ContractServiceFacility,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, contract_id: ContractId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ContractWithProvidersUnion]:
        """
        Parameters
        ----------
        contract_id : ContractId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ContractWithProvidersUnion]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/contracts/v3/{jsonable_encoder(contract_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractWithProvidersUnion,
                parse_obj_as(
                    type_=ContractWithProvidersUnion,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        type: typing.Optional[ContractType] = None,
        contracting_provider_id: typing.Optional[ContractingProviderId] = None,
        rendering_provider_ids: typing.Optional[
            typing.Union[RenderingProviderid, typing.Sequence[RenderingProviderid]]
        ] = None,
        payer_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        states: typing.Optional[typing.Union[State, typing.Sequence[State]]] = None,
        contract_status: typing.Optional[ContractStatus] = None,
        sort: typing.Optional[ContractSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ContractsPage]:
        """
        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Max number of contracts returned. Defaults to 1000. Max is 1000.

        type : typing.Optional[ContractType]
            The type of contract

        contracting_provider_id : typing.Optional[ContractingProviderId]

        rendering_provider_ids : typing.Optional[typing.Union[RenderingProviderid, typing.Sequence[RenderingProviderid]]]

        payer_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter to contracts that include any of the included payer names.

        states : typing.Optional[typing.Union[State, typing.Sequence[State]]]

        contract_status : typing.Optional[ContractStatus]
            The status of the contract. Defaults to `pending`

        sort : typing.Optional[ContractSortField]
            Potentially sort by a contract related attribute.  Defaults to created_at

        sort_direction : typing.Optional[SortDirection]
            Direction of sort, defaulting to desc

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ContractsPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/contracts/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "page_token": page_token,
                "limit": limit,
                "type": type,
                "contracting_provider_id": contracting_provider_id,
                "rendering_provider_ids": rendering_provider_ids,
                "payer_names": payer_names,
                "states": states,
                "contract_status": contract_status,
                "sort": sort,
                "sort_direction": sort_direction,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractsPage,
                parse_obj_as(
                    type_=ContractsPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, request: ContractCreateUnion, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ContractWithProvidersUnion]:
        """
        Creates a new contract within the user's current organization

        Parameters
        ----------
        request : ContractCreateUnion

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ContractWithProvidersUnion]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/contracts/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractWithProvidersUnion,
                parse_obj_as(
                    type_=ContractWithProvidersUnion,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, contract_id: ContractId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        contract_id : ContractId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/contracts/v3/{jsonable_encoder(contract_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "ContractIsLinkedToFeeScheduleHttpError":
                raise ContractIsLinkedToFeeScheduleHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ContractIsLinkedToFeeScheduleError,
                        parse_obj_as(
                            type_=ContractIsLinkedToFeeScheduleError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        contract_id: ContractId,
        *,
        request: ContractUpdateUnion,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ContractWithProvidersUnion]:
        """
        Parameters
        ----------
        contract_id : ContractId

        request : ContractUpdateUnion

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ContractWithProvidersUnion]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/contracts/v3/{jsonable_encoder(contract_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractWithProvidersUnion,
                parse_obj_as(
                    type_=ContractWithProvidersUnion,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "ContractInvalidExpirationDateHttpError":
                raise ContractInvalidExpirationDateHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ContractInvalidExpirationDateError,
                        parse_obj_as(
                            type_=ContractInvalidExpirationDateError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_contract_service_facility(
        self,
        contract_id: ContractId,
        *,
        service_facility_id: uuid.UUID,
        provider_ids: typing.Set[uuid.UUID],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ContractServiceFacility]:
        """
        Parameters
        ----------
        contract_id : ContractId

        service_facility_id : uuid.UUID
            The UUID of the service facility

        provider_ids : typing.Set[uuid.UUID]
            The providers who are authorized under the contract

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ContractServiceFacility]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/contracts/v3/{jsonable_encoder(contract_id)}/service-facilities",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json={
                "service_facility_id": service_facility_id,
                "provider_ids": provider_ids,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractServiceFacility,
                parse_obj_as(
                    type_=ContractServiceFacility,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_contract_service_facility(
        self,
        contract_id: ContractId,
        contract_service_facility_id: ContractServiceFacilityId,
        *,
        service_facility_id: typing.Optional[uuid.UUID] = OMIT,
        provider_ids: typing.Optional[typing.Set[uuid.UUID]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ContractServiceFacility]:
        """
        Parameters
        ----------
        contract_id : ContractId

        contract_service_facility_id : ContractServiceFacilityId

        service_facility_id : typing.Optional[uuid.UUID]
            The UUID of the service facility

        provider_ids : typing.Optional[typing.Set[uuid.UUID]]
            The providers who are authorized under the contract

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ContractServiceFacility]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/contracts/v3/{jsonable_encoder(contract_id)}/service-facilities/{jsonable_encoder(contract_service_facility_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "service_facility_id": service_facility_id,
                "provider_ids": provider_ids,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                ContractServiceFacility,
                parse_obj_as(
                    type_=ContractServiceFacility,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .user_configured_prompts_result import UserConfiguredPromptsResult


class UserConfiguredPromptsRecommendationPayload(UniversalBaseModel):
    """
    Payload for user-configured prompt recommendations from AI analysis
    """

    results: typing.List[UserConfiguredPromptsResult] = pydantic.Field()
    """
    Array of results from all enabled user-configured prompts
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .schema import Schema


class SchemaGetMultiResponse(UniversalBaseModel):
    """
    Examples
    --------
    import uuid

    from candid.resources.commons import Primitive
    from candid.resources.custom_schemas.resources.v_1 import (
        Schema,
        SchemaField,
        SchemaGetMultiResponse,
    )

    SchemaGetMultiResponse(
        schemas=[
            Schema(
                id=uuid.UUID(
                    "ec096b13-f80a-471d-aaeb-54b021c9d582",
                ),
                name="General Medicine",
                description="Values associated with a generic visit",
                fields=[
                    SchemaField(
                        key="provider_category",
                        type=Primitive.STRING,
                    ),
                    SchemaField(
                        key="is_urgent_care",
                        type=Primitive.BOOLEAN,
                    ),
                    SchemaField(
                        key="bmi",
                        type=Primitive.DOUBLE,
                    ),
                    SchemaField(
                        key="age",
                        type=Primitive.INTEGER,
                    ),
                ],
            )
        ],
    )
    """

    schemas: typing.List[Schema]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
import uuid

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .allocation_earmark_type import AllocationEarmarkType


class BalanceEarmark(UniversalBaseModel):
    """
    Represents an active balance earmarking record that holds allocated funds for future auto-allocation.
    Earmarks are created when funds are allocated but should be held for a specific encounter or date of service.
    Only active (non-deleted) earmarks are returned.
    """

    id: uuid.UUID
    target: AllocationEarmarkType = pydantic.Field()
    """
    The target for this earmark (date of service or external encounter ID)
    """

    amount_earmarked_cents: typing.Optional[int] = pydantic.Field(default=None)
    """
    The amount earmarked in cents for future allocation
    """

    created_by_allocation_id: typing.Optional[uuid.UUID] = pydantic.Field(default=None)
    """
    The ID of the allocation that created this earmark
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .property_casualty_patient_identifier_qualifier import PropertyCasualtyPatientIdentifierQualifier


class PropertyCasualtyPatientIdentifier(UniversalBaseModel):
    property_casualty_patient_identifier_qualifier: PropertyCasualtyPatientIdentifierQualifier = pydantic.Field()
    """
    Represents REF01 on the EDI 837 Loop 2010CA Property and Casualty Patient Identifier segment.
    Valid values include 1W (Member Identification Number) and SY (Social Security Number).
    """

    property_casualty_patient_identifier: str = pydantic.Field()
    """
    Represents REF02 on the EDI 837 Loop 2010CA Property and Casualty Patient Identifier segment. Value cannot exceed 50 characters
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .coverage_association import CoverageAssociation
from .image_status import ImageStatus
from .patient_association import PatientAssociation


class MutableImage(UniversalBaseModel):
    """
    An object representing an Image.
    """

    file_name: str
    display_name: str
    file_type: str
    status: ImageStatus
    coverage: typing.Optional[CoverageAssociation] = None
    patient: typing.Optional[PatientAssociation] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
